import numpy as np
import os


if __name__ == '__main__':

    # specify the grid of parameters where we want to perform the simulation on
    pot_cut_values = np.array([2.5, 3.0, 3.5, 4.0, 4.5, 5.0]) #in angstroms
    ecutwfc_values = np.array([50.0, 60.0, 70.0, 80.0, 90.0, 100.0]) #50-100,
    cell_size_values = np.array([10.0, 11.25, 12.5, 13.75, 15.0]) #in angstroms

    # remove old files
    os.system(str("rm -f total_energies.out"))
    os.system(str("rm -f xc_energies.out"))
    os.system(str("rm -f parameters.out"))
    os.system(str("rm -rf pseudopotentials"))
    os.system(str("rm -f generate_pseudo_simple.py"))
    os.system(str("rm -f hookium.pwi"))
              
    # copy new versions of the pseudopotential-generation files into this folder
    os.system(str("cp ../Template_pseudo_potential_generation/generate_pseudo_simple.py ."))
    os.system(str("cp -r ../Template_pseudo_potential_generation/pseudopotentials ."))
    
    #create a folder for the plots
    os.system(str("mkdir -p plots"))


    for pot_cut in pot_cut_values:
        for ecutwfc in ecutwfc_values:
            for cell_size in cell_size_values:
                # generate the pseudopotential according to the pot_cut-value and cell-size-value
                os.system(str("python3 ./generate_pseudo_simple.py " + str(pot_cut ) + " " + str(cell_size)))
                if(pot_cut <= cell_size/2.5): #only perform the calculation if the potential pot_cut is small enough, s.t. the potential is zero at the cell-boundary
                    print("Performing the simulation with pot_cut = " + str(pot_cut) + ", ecutwfc = " + str(ecutwfc) + " and cell_size = " + str(cell_size))

                    # Adapt the pw.x input file according to the parameters
                    input_file = open("../Template_PW_Input_File/hookium.pwi")
                    string_list = input_file.readlines()
                    input_file.close()

                    string_list[6]  = str("   disk_io          = 'low'\n")
                    string_list[15] = str("   ecutwfc          = " + str(ecutwfc) + "\n")
                    string_list[16] = str("   ecutrho          = " + str(4*ecutwfc) + "\n")
                    string_list[38] = str(str(cell_size) + " 0.00000000000000 0.00000000000000 \n")
                    string_list[39] = str("0.00000000000000 " + str(cell_size) + " 0.00000000000000 \n")
                    string_list[40] = str("0.00000000000000 0.00000000000000 " +str(cell_size) +  "\n")
                    string_list[43] = str("He " + str(cell_size*0.5) + " " + str(cell_size*0.5) + " " + str(cell_size*0.5) +  "\n")

                    input_file = open("./hookium.pwi", "w")
                    new_file_contents = "".join(string_list)
                    input_file.write(new_file_contents)
                    input_file.close()

                    # perform the actual simulation
                    os.system(str("mpirun -n 4 pw.x -i hookium.pwi > file.out")) #do the calculation with the given parameters

                    #write the parameters into a file
                    os.system(str("echo " + str(str(pot_cut) + " " + str(ecutwfc) + " " + str(cell_size)) +  " >> parameters.out"))

                    #write the corresponding total energies into a file
                    os.system(str("cat file.out | grep '\!'  | tail -1 | awk '{print $5}' >> total_energies.out")) # write the corresponding xc energies into a file
                    os.system(str("cat file.out | grep '\\xc contribution'  | tail -1 | awk '{print $4}' >> xc_energies.out"))
                    
                    
    #clean up
    os.system(str("rm -f generate_pseudo_simple.py"))
    os.system(str("rm -rf pseudopotentials"))
    os.system(str("rm hookium.pwi"))
