import numpy as np
import matplotlib.pyplot as plt

fontsize = 15

# specify the grid of parameters where we want to perform the simulation on
pot_cut_values = np.array([2.5, 3.0, 3.5, 4.0, 4.5, 5.0]) #in angstroms
ecutwfc_values = np.array([50.0, 60.0, 70.0, 80.0, 90.0, 100.0]) #50-100,
cell_size_values = np.array([10.0, 11.25, 12.5, 13.75, 15.0]) #in angstroms
xc_energies = np.loadtxt("./results/xc_energies.out")


num_elems = len(ecutwfc_values)*len(cell_size_values)


different_plots  = np.empty(num_elems, dtype = object)
different_x_axis = np.empty(num_elems, dtype = object)
absolute_errors  = np.empty(num_elems, dtype = object)

for i in range(num_elems):
    different_plots[i]  = []
    different_x_axis[i] = []
    absolute_errors[i]  = []


count = 0
for i, pot_cut in enumerate(pot_cut_values):
    for j in range(len(ecutwfc_values)):
        for k, cell_size in enumerate(cell_size_values):
            if(pot_cut <= cell_size/2.5):
                different_plots[j*len(cell_size_values)+k].append(xc_energies[count])
                different_x_axis[j*len(cell_size_values)+k].append(pot_cut)
                count+=1



true_value = (-0.493-0.051)*2
true_value_min = (-0.4935-0.0515)*2
true_value_max = (-0.4925-0.0505)*2

plt.figure(figsize = (15,10))
for j, ecutwf in enumerate(ecutwfc_values):
    for k, cell_size in enumerate(cell_size_values):
        # if(cell_size == 7.5 and (ecutwf == 50 or ecutwf == 75 or ecutwf == 100)):
        #     continue
        plt.plot(different_x_axis[j*len(cell_size_values)+k], different_plots[j*len(cell_size_values)+k], "-o", label = str("cell size = " + str(cell_size) + ", ecutwf = " + str(ecutwf)))
#plt.plot(np.linspace(2.5, 5.0), true_value*np.linspace(2.5, 10.0)**0, label = "true value")
#plt.plot(np.linspace(2.5, 5.0), true_value_min*np.linspace(2.5, 10.0)**0, label = "true value min")
#plt.plot(np.linspace(2.5, 5.0), true_value_max*np.linspace(2.5, 10.0)**0, label = "true value max")
#plt.ylim(())
plt.xlabel("pot_cut size of potential in angstrom")
plt.ylabel("xc energy in Ry")
plt.legend()
plt.grid(True)
plt.savefig("plots/Convergence_Plot.png")
plt.show()


y1 = np.zeros(len(pot_cut_values))
y2 = np.ones(len(pot_cut_values))

count = 0
for i, pot_cut in enumerate(pot_cut_values):
    for j in range(len(ecutwfc_values)):
        for k, cell_size in enumerate(cell_size_values):
            if(pot_cut <= cell_size/2.5):
                absolute_errors[j*len(cell_size_values)+k].append(max(abs(different_plots[j*len(cell_size_values)+k][i]- different_plots[len(ecutwfc_values)*len(cell_size_values)-1][-1]), 2e-8))
                y1[i] = max(y1[i], absolute_errors[j*len(cell_size_values)+k][i])
                y2[i] = min(y2[i], absolute_errors[j*len(cell_size_values)+k][i])


plt.figure(figsize = (15,10))
for j, ecutwf in enumerate(ecutwfc_values):
    for k, cell_size in enumerate(cell_size_values):
        #if(cell_size != 10.0 or ecutwf!=90):
            #continue
        plt.semilogy(different_x_axis[j*len(cell_size_values)+k], absolute_errors[j*len(cell_size_values)+k], "-o", label = str("cell size = " + str(cell_size) + ", ecutwf = " + str(ecutwf)))
plt.xlabel("pot_cut size of potential in angstrom")
plt.ylabel("Absolute error in Ry")
plt.legend()
plt.grid(True)
plt.ylim((2e-8, 1e-3))
plt.savefig("plots/Convergence_Plot_Absolute_Error.png")
plt.show()


plt.rcParams.update({'font.size': fontsize})
plt.rc('axes', axisbelow=True)
plt.figure(figsize = (10,10))
plt.title("Convergence Analysis")
#for j, ecutwf in enumerate(ecutwfc_values):
    #for k, cell_size in enumerate(cell_size_values):
        #if(cell_size != 10.0 or ecutwf!=90):
            #continue
        #plt.semilogy(different_x_axis[j*len(cell_size_values)+k], absolute_errors[j*len(cell_size_values)+k], "-o")#, label = str("cell size = " + str(cell_size) + ", ecutwf = " + str(ecutwf)))
#plt.semilogy(pot_cut_values, y1, label = "y1")
#plt.semilogy(pot_cut_values, y2, label = "y2")
plt.grid(True, which='major')
plt.grid(True, which='minor')
plt.fill_between(pot_cut_values, y1, y2, color = "lightsteelblue")
plt.xlabel("Position of the cutoff of the potential in angstrom")
plt.ylabel("Absolute error of the xc-energy compared to the most converged xc-energy in Ry")
#plt.legend()
plt.ylim((2e-8, 1e-3))
plt.yscale("log")
plt.xlim(pot_cut_values[0], pot_cut_values[-1])
plt.savefig("plots/Convergence_Plot_Absolute_Error_Fill_Between.png")
plt.show()
