import numpy as np
import matplotlib.pyplot as plt
from scipy.special import erf
import scipy.integrate as integrate
from scipy.misc import derivative
import seaborn as sns

alat = 30.0*1.889726125 #in bohr radii

bottom_of_potential = -2.8291185532e+00/2.0 #convert to Hartree

k = (0.1)**2
omega = np.sqrt(k)
reference_total_energy  = 0.5
reference_HOMO          = 0.35
reference_LUMO          = 1.0594492-reference_total_energy
reference_LUMO_plus_one = 1.0947376-reference_total_energy

# Specify whether you want to plot the results
plot                      = True

# Specify which quanties you want to plot
results_exact              = True
results_LDA                = True
results_PBE                = True
results_DSCF               = True
results_KI                 = True
results_PZ                 = True
results_KIPZ               = True
results_exact_empty_states = True
results_LDA_empty_states   = True
results_PBE_empty_states   = True
results_DSCF_empty_states  = True
results_PZ_empty_states    = True
results_KI_empty_states    = True
results_KIPZ_empty_states  = True
compute_difference_to_KIPZ = True

# Specifications for nicer plots
bottom                 = 0.13
left_two_after_point   = 0.18
left_three_after_point = 0.215
right                  = 0.97
top                    = 0.98
resolution             = 1000
fig_size               = 246.0/72.27 
sns.set(style = "whitegrid", context = "paper",  rc={'text.usetex' : True, "font.family": 'Times New Roman'}) #, "font.serif": "Computer Modern"})
sns.set_palette(sns.color_palette("bright"))


list_for_densities             = []
list_for_xc_pots               = []
list_for_xc_appl_pots          = []
list_for_xc_appl_scaled_pots   = []
list_for_x_pots                = []
list_for_c_pots                = []
list_for_bare_pots             = []

labels_for_densities           = []
labels_for_xc_pots             = []
labels_for_xc_appl_pots        = []
labels_for_xc_appl_scaled_pots = []
labels_for_x_pots              = []
labels_for_c_pots              = []
labels_for_bare_pots           = []


linestyles_for_densities       = []
linestyles_for_xc_potential    = []

# integrates a radial symmetric function defined on an equidistand line x over all space
def simple_integrate(x,y):
    dx = x[1]-x[0]
    return dx*4*np.pi*np.sum(x**2*y)

const = 4/(25*np.pi*(61*np.pi+48*np.sqrt(5*np.pi)))

def density(r):
    return const/(800)*(200*(45+r**2)*np.exp(-r**2/5)+np.exp(-r**2/10)*np.sqrt(10*np.pi)*(2875+190*r**2+r**4+20*(175/r+50*r+r**3)*erf(r/(np.sqrt(10)))))


def v_x(r):
    infty = 1000
    I_1 = integrate.quad(lambda s: density(s)*s**2, 0, r)[0]
    I_2 = integrate.quad(lambda s: density(s)*s, r, infty)[0]
    return -0.5*4*np.pi*(1.0/r*I_1+I_2)

def e_KS(r):
    return 0.35

def v_ext(r):
    return 0.5*k*r**2

def phi_KS(r):
    return np.sqrt(0.5*density(r))

def v_KE(r):
    dx = 1e-5
    order = 3
    return -(1.0/r*derivative(phi_KS, r, dx=dx, n=1, order=order)+0.5*derivative(phi_KS, r, dx=dx, n=2, order=order))/phi_KS(r)

def v_xc(r):
    return e_KS(r)-v_ext(r)+2*v_x(r)-v_KE(r)

if(results_exact == True):
    exact_name = "exact"

    number_of_points = 1000
    rs = np.linspace(1e-4,alat/2.0, number_of_points)
    
    # compute the quanties of interest
    exact_density                 = np.zeros((number_of_points, 2))
    exact_xc_potential            = np.zeros((number_of_points, 2))
    exact_x_potential             = np.zeros((number_of_points, 2))
    exact_c_potential             = np.zeros((number_of_points, 2))
    exact_xc_applied              = np.zeros_like(exact_density)
    exact_xc_applied_scaled       = np.zeros_like(exact_density)
    exact_density[:,0]            = rs
    exact_xc_potential[:,0]       = rs
    exact_x_potential[:,0]        = rs
    exact_c_potential[:,0]        = rs
    exact_xc_applied[:,0]         = rs
    exact_xc_applied_scaled[:,0]  = rs
    for i, r in enumerate(rs):
        exact_density[i,1]          = density(rs[i])
        exact_xc_potential[i,1]     = v_xc(rs[i])
        exact_x_potential[i,1]      = v_x(rs[i])
        
    exact_c_potential[:,1]        = exact_xc_potential[:,1]-exact_x_potential[:,1] 
    exact_xc_applied[:,1]         = exact_xc_potential[:,1]*np.sqrt(0.5*exact_density[:,1])
    exact_xc_applied_scaled[:,1]  = 4*np.pi*(exact_density[:,0])**2*exact_xc_potential[:,1]*0.5*exact_density[:,1]
    
    #add to the lists to plot
    list_for_densities.append(exact_density)
    labels_for_densities.append(exact_name)
    linestyles_for_densities.append('solid')
    linestyles_for_xc_potential.append('solid')
    list_for_xc_pots.append(exact_xc_potential)
    labels_for_xc_pots.append(exact_name)
    list_for_x_pots.append(exact_x_potential)
    labels_for_x_pots.append(exact_name)
    list_for_c_pots.append(exact_c_potential)
    labels_for_c_pots.append(exact_name)
    list_for_xc_appl_pots.append(exact_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{exact})\phi_{KS}^{exact}$")
    list_for_xc_appl_scaled_pots.append(exact_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{exact})n_{HOMO}^{exact}$")
    
if(results_LDA == True):
    LDA_name = "LDA"
    # load the results
    LDA_density                   = np.loadtxt("../Analysis_LDA/postprocessing/results/hookium_density.txt")
    LDA_pot_bare_H_xc             = np.loadtxt("../Analysis_LDA/postprocessing/results/hookium_bare_H_xc.txt")
    LDA_pot_bare_H                = np.loadtxt("../Analysis_LDA/postprocessing/results/hookium_bare_H.txt")
    LDA_pot_only_x_bare_H_x       = np.loadtxt("../Analysis_LDA/postprocessing/results/hookium_bare_H_x.txt")
    LDA_pot_only_x_bare_H         = np.loadtxt("../Analysis_LDA/postprocessing/results/hookium_only_x_bare_H.txt")
    LDA_xc_applied                = np.zeros_like(LDA_density)
    LDA_xc_applied_scaled         = np.zeros_like(LDA_density)
    # Scale to the cell size
    LDA_density[:,0]             *= alat
    LDA_pot_bare_H_xc[:,0]       *= alat
    LDA_pot_bare_H[:,0]          *= alat
    LDA_pot_only_x_bare_H_x[:,0] *= alat
    LDA_pot_only_x_bare_H[:,0]   *= alat
    LDA_xc_applied[:,0]           = LDA_density[:,0]
    LDA_xc_applied_scaled[:,0]    = LDA_density[:,0]
    # Potentials are given in Rydberg -> convert to Hartree
    LDA_pot_bare_H_xc[:,1]       /= 2.0
    LDA_pot_bare_H[:,1]          /= 2.0
    LDA_pot_only_x_bare_H_x[:,1] /= 2.0
    LDA_pot_only_x_bare_H[:,1]   /= 2.0
    # compute the quanties of interest
    LDA_xc_potential              = np.zeros_like(LDA_density)
    LDA_x_potential               = np.zeros_like(LDA_density)
    LDA_c_potential               = np.zeros_like(LDA_density)
    LDA_xc_potential[:,0]         = LDA_density[:,0]
    LDA_x_potential[:,0]          = LDA_density[:,0]
    LDA_c_potential[:,0]          = LDA_density[:,0]
    LDA_xc_potential[:,1]         = LDA_pot_bare_H_xc[:,1] - LDA_pot_bare_H[:,1]
    LDA_x_potential[:,1]          = LDA_pot_only_x_bare_H_x[:,1] - LDA_pot_only_x_bare_H[:,1]
    LDA_c_potential[:,1]          = LDA_xc_potential[:,1] - LDA_x_potential[:,1] 
    LDA_xc_applied[:,1]           = LDA_xc_potential[:,1]*np.sqrt(0.5*LDA_density[:,1])
    LDA_xc_applied_scaled[:,1]    = 4*np.pi*(LDA_density[:,0])**2*LDA_xc_potential[:,1]*0.5*LDA_density[:,1]
    #add to the lists to plot
    list_for_densities.append(LDA_density)
    labels_for_densities.append(LDA_name)
    linestyles_for_densities.append('solid')
    linestyles_for_xc_potential.append('solid')
    list_for_xc_pots.append(LDA_xc_potential)
    labels_for_xc_pots.append(LDA_name)
    list_for_x_pots.append(LDA_x_potential)
    labels_for_x_pots.append(LDA_name)
    list_for_c_pots.append(LDA_c_potential)
    labels_for_c_pots.append(LDA_name)
    list_for_xc_appl_pots.append(LDA_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{LDA})\phi_{KS}^{LDA}$")
    list_for_xc_appl_scaled_pots.append(LDA_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{LDA})n_{HOMO}^{LDA}$")
    
if(results_PBE == True):
    PBE_name = "PBE"
    # load the results
    PBE_density                   = np.loadtxt("../Analysis_PBE/postprocessing/results/hookium_density.txt")
    PBE_pot_bare_H_xc             = np.loadtxt("../Analysis_PBE/postprocessing/results/hookium_bare_H_xc.txt")
    PBE_pot_bare_H                = np.loadtxt("../Analysis_PBE/postprocessing/results/hookium_bare_H.txt")
    PBE_pot_only_x_bare_H_x       = np.loadtxt("../Analysis_PBE/postprocessing/results/hookium_bare_H_x.txt")
    PBE_pot_only_x_bare_H         = np.loadtxt("../Analysis_PBE/postprocessing/results/hookium_only_x_bare_H.txt")
    HOMO_energy_PBE               = np.loadtxt("../Analysis_PBE/postprocessing/results/eigenvalues.out")[0]
    PBE_xc_applied                = np.zeros_like(PBE_density)
    PBE_xc_applied_scaled         = np.zeros_like(PBE_density)
    # Scale to the cell size
    PBE_density[:,0]             *= alat
    PBE_pot_bare_H_xc[:,0]       *= alat
    PBE_pot_bare_H[:,0]          *= alat
    PBE_pot_only_x_bare_H_x[:,0] *= alat
    PBE_pot_only_x_bare_H[:,0]   *= alat
    PBE_xc_applied[:,0]           = PBE_density[:,0]
    PBE_xc_applied_scaled[:,0]    = PBE_density[:,0]
    # Potentials are given in Rydberg -> convert to Hartree
    PBE_pot_bare_H_xc[:,1]       /= 2.0
    PBE_pot_bare_H[:,1]          /= 2.0
    PBE_pot_only_x_bare_H_x[:,1] /= 2.0
    PBE_pot_only_x_bare_H[:,1]   /= 2.0
    # compute the quanties of interest
    PBE_xc_potential              = np.zeros_like(PBE_density)
    PBE_x_potential               = np.zeros_like(PBE_density)
    PBE_c_potential               = np.zeros_like(PBE_density)
    PBE_xc_potential[:,0]         = PBE_density[:,0]
    PBE_x_potential[:,0]          = PBE_density[:,0]
    PBE_c_potential[:,0]          = PBE_density[:,0]
    PBE_xc_potential[:,1]         = PBE_pot_bare_H_xc[:,1] - PBE_pot_bare_H[:,1]
    PBE_x_potential[:,1]          = PBE_pot_only_x_bare_H_x[:,1] - PBE_pot_only_x_bare_H[:,1]
    PBE_c_potential[:,1]          = PBE_xc_potential[:,1] - PBE_x_potential[:,1] 
    PBE_xc_applied[:,1]           = PBE_xc_potential[:,1]*np.sqrt(0.5*PBE_density[:,1])
    PBE_xc_applied_scaled[:,1]    = 4*np.pi*(PBE_density[:,0])**2*PBE_xc_potential[:,1]*0.5*PBE_density[:,1]
    #add to the lists to plot
    list_for_densities.append(PBE_density)
    labels_for_densities.append(PBE_name)
    linestyles_for_densities.append('solid')
    linestyles_for_xc_potential.append('solid')
    list_for_xc_pots.append(PBE_xc_potential)
    labels_for_xc_pots.append(PBE_name)
    list_for_x_pots.append(PBE_x_potential)
    labels_for_x_pots.append(PBE_name)
    list_for_c_pots.append(PBE_c_potential)
    labels_for_c_pots.append(PBE_name)
    list_for_xc_appl_pots.append(PBE_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{PBE})\phi_{KS}^{PBE}$")
    list_for_xc_appl_scaled_pots.append(PBE_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{PBE})n_{HOMO}^{PBE}$")
    
    print("HOMO energy PBE                   = {:10.8f} Hartree".format(HOMO_energy_PBE-bottom_of_potential))
    print()
    
    if(results_DSCF == True):
        total_energy_PBE          = np.loadtxt("../Analysis_PBE/postprocessing/results/total_energy.out")/2.0
        total_energy_PBE_Ion      = np.loadtxt("../Analysis_PBE_Ion/postprocessing/results/total_energy.out")/2.0
        
        print("HOMO energy DSCF with PBE         = {:10.8f} Hartree".format((total_energy_PBE-bottom_of_potential)-total_energy_PBE_Ion))
        print()
        
if(results_PZ == True):
    PZ_name = "PZ"
    # load the results
    PZ_density                   = np.loadtxt("../Analysis_KIPZ/postprocessing_pz/results/hookium_density.txt")
    PZ_pot_bare_H_xc             = np.loadtxt("../Analysis_KIPZ/postprocessing_pz/results/hookium_bare_H_xc.txt")
    PZ_pot_bare_H                = np.loadtxt("../Analysis_KIPZ/postprocessing_pz/results/hookium_bare_H.txt")
    PZ_v_ki                      = np.loadtxt("../Analysis_KIPZ/postprocessing_pz/results/hookium_v_ki.txt")
    HOMO_energy_PZ_DFT           = np.loadtxt("../Analysis_KIPZ/postprocessing/results/HOMO_PZ.out")
    PZ_xc_potential              = np.zeros_like(PZ_density)
    PZ_xc_applied                = np.zeros_like(PZ_density)
    PZ_xc_applied_scaled         = np.zeros_like(PZ_density)
    # Scale to the cell size
    PZ_density[:,0]             *= alat
    PZ_v_ki[:,0]                *= alat
    PZ_xc_applied[:,0]           = PZ_density[:,0]
    PZ_xc_applied_scaled[:,0]    = PZ_density[:,0]
    # Potentials are given in Rydberg -> convert to Hartree
    PZ_pot_bare_H_xc[:,1]       /= 2.0
    PZ_pot_bare_H[:,1]          /= 2.0
    # compute the quanties of interest
    PZ_xc_potential              = np.zeros_like(PZ_density)
    PZ_xc_potential[:,0]         = PZ_density[:,0]
    PZ_xc_potential[:,1]         = PZ_pot_bare_H_xc[:,1] - PZ_pot_bare_H[:,1] + PZ_v_ki[:,1]
    PZ_xc_applied[:,1]           = PZ_xc_potential[:,1]*np.sqrt(0.5*PZ_density[:,1])
    PZ_xc_applied_scaled[:,1]    = 4*np.pi*(PZ_density[:,0])**2*PZ_xc_potential[:,1]*0.5*PZ_density[:,1]
    #add to the lists to plot
    list_for_densities.append(PZ_density)
    labels_for_densities.append(PZ_name)
    linestyles_for_densities.append('solid')
    linestyles_for_xc_potential.append('solid')
    list_for_xc_pots.append(PZ_xc_potential)
    labels_for_xc_pots.append(r"$v_{xc}^{PBE}+v^{PZ}$")
    list_for_xc_appl_pots.append(PZ_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{PBE}+v^{PZ})\phi_{KS}^{PZ}$")
    list_for_xc_appl_scaled_pots.append(PZ_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{PBE}+v^{PZ})n_{HOMO}^{PZ}$")
    
    print("HOMO energy DFT with PZ-Density   = {:10.8f} Hartree".format(HOMO_energy_PZ_DFT-bottom_of_potential))
    print()

if(results_KI == True):
    KI_name = "KI"
    # load the results
    KI_density                   = np.loadtxt("../Analysis_KI/postprocessing/results/hookium_density.txt")
    KI_pot_bare_H_xc             = np.loadtxt("../Analysis_KI/postprocessing/results/hookium_bare_H_xc.txt")
    KI_pot_bare_H                = np.loadtxt("../Analysis_KI/postprocessing/results/hookium_bare_H.txt")
    KI_v_ki                      = np.loadtxt("../Analysis_KI/postprocessing/results/hookium_v_ki.txt")
    HOMO_energy_KI               = np.loadtxt("../Analysis_KI/postprocessing/results/HOMO.out")
    HOMO_energy_KI_DFT           = np.loadtxt("../Analysis_KI/postprocessing/results/HOMO_KI_DFT.out")
    KI_xc_potential              = np.zeros_like(KI_density)
    KI_xc_applied                = np.zeros_like(KI_density)
    KI_xc_applied_scaled         = np.zeros_like(KI_density)
    # Scale to the cell size
    KI_density[:,0]             *= alat
    KI_pot_bare_H_xc[:,0]       *= alat
    KI_pot_bare_H[:,0]          *= alat
    KI_v_ki[:,0]                *= alat
    KI_xc_potential[:,0]         = KI_density[:,0]
    KI_xc_applied[:,0]           = KI_density[:,0]
    KI_xc_applied_scaled[:,0]    = KI_density[:,0]
    # Potentials are given in Rydberg -> convert to Hartree
    KI_pot_bare_H_xc[:,1]       /= 2.0
    KI_pot_bare_H[:,1]          /= 2.0
    # compute the quanties of interest
    KI_xc_potential[:,1]         = KI_pot_bare_H_xc[:,1] - KI_pot_bare_H[:,1] + KI_v_ki[:,1]
    KI_xc_applied[:,1]           = KI_xc_potential[:,1]*np.sqrt(0.5*KI_density[:,1])
    KI_xc_applied_scaled[:,1]    = 4*np.pi*(KI_density[:,0])**2*KI_xc_potential[:,1]*0.5*KI_density[:,1]
    #add to the lists to plot
    list_for_densities.append(KI_density)
    labels_for_densities.append(KI_name)
    linestyles_for_densities.append('dashdot')
    linestyles_for_xc_potential.append('solid')
    list_for_xc_pots.append(KI_xc_potential)
    labels_for_xc_pots.append(r"$v_{xc}^{PBE}+v^{KI}$")
    list_for_xc_appl_pots.append(KI_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{PBE}+v^{KI})\phi_{KS}^{KI}$")
    list_for_xc_appl_scaled_pots.append(KI_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{PBE}+v^{KI})n_{HOMO}^{KI}$")
    
    alpha_Pi_KI = 0.5*simple_integrate(KI_v_ki[:,0], KI_v_ki[:,1]*KI_density[:,1])
    
    print("HOMO energy DFT with KI-Density   = {:10.8f} Hartree".format(HOMO_energy_KI_DFT-bottom_of_potential))
    print("HOMO energy KI                    = {:10.8f} Hartree".format(HOMO_energy_KI-bottom_of_potential))
    print("Difference between the two        = {:10.7f} Hartree".format((HOMO_energy_KI-bottom_of_potential)-(HOMO_energy_KI_DFT-bottom_of_potential)))
    print("alpha*Pi_KI                       = {:10.7f} Hartree".format(alpha_Pi_KI))
    print()
    
    
if(results_KIPZ == True):
    KIPZ_name = "KIPZ"
    # load the results
    KIPZ_density                   = np.loadtxt("../Analysis_KIPZ/postprocessing/results/hookium_density.txt")
    KIPZ_pot_bare_H_xc             = np.loadtxt("../Analysis_KIPZ/postprocessing/results/hookium_bare_H_xc.txt")
    KIPZ_pot_bare_H                = np.loadtxt("../Analysis_KIPZ/postprocessing/results/hookium_bare_H.txt")
    KIPZ_v_ki                      = np.loadtxt("../Analysis_KIPZ/postprocessing/results/hookium_v_ki.txt")
    HOMO_energy_KIPZ               = np.loadtxt("../Analysis_KIPZ/postprocessing/results/HOMO.out")
    HOMO_energy_KIPZ_DFT           = np.loadtxt("../Analysis_KIPZ/postprocessing/results/HOMO_KIPZ_DFT.out")
    KIPZ_xc_potential              = np.zeros_like(KIPZ_density)
    KIPZ_xc_applied                = np.zeros_like(KIPZ_density)
    KIPZ_xc_applied_scaled         = np.zeros_like(KIPZ_density)
    # Scale to the cell size
    KIPZ_density[:,0]             *= alat
    KIPZ_pot_bare_H_xc[:,0]       *= alat
    KIPZ_pot_bare_H[:,0]          *= alat
    KIPZ_v_ki[:,0]                *= alat
    KIPZ_xc_potential[:,0]         = KIPZ_density[:,0]
    KIPZ_xc_applied[:,0]           = KIPZ_density[:,0]
    KIPZ_xc_applied_scaled[:,0]    = KIPZ_density[:,0]
    # Potentials are given in Rydberg -> convert to Hartree
    KIPZ_pot_bare_H_xc[:,1]       /= 2.0
    KIPZ_pot_bare_H[:,1]          /= 2.0
    # compute the quanties of interest
    KIPZ_xc_potential[:,1]         = KIPZ_pot_bare_H_xc[:,1] - KIPZ_pot_bare_H[:,1] + KIPZ_v_ki[:,1]
    KIPZ_xc_applied[:,1]           = KIPZ_xc_potential[:,1]*np.sqrt(0.5*KIPZ_density[:,1])
    KIPZ_xc_applied_scaled[:,1]    = 4*np.pi*(KIPZ_density[:,0])**2*KIPZ_xc_potential[:,1]*0.5*KIPZ_density[:,1]   
    # add to the lists to plot
    list_for_densities.append(KIPZ_density)
    labels_for_densities.append(KIPZ_name)
    linestyles_for_densities.append('dashdot') 
    linestyles_for_xc_potential.append('dashdot')
    list_for_xc_pots.append(KIPZ_xc_potential)
    labels_for_xc_pots.append(r"$v_{xc}^{PBE}+v^{KIPZ}$")
    list_for_xc_appl_pots.append(KIPZ_xc_applied)
    labels_for_xc_appl_pots.append(r"$(v_{xc}^{PBE}+v^{KIPZ})\phi_{KS}^{KIPZ}$")
    list_for_xc_appl_scaled_pots.append(KIPZ_xc_applied_scaled)
    labels_for_xc_appl_scaled_pots.append(r"$4\pi r^2(v_{xc}^{PBE}+v^{KIPZ})n_{HOMO}^{KIPZ}$")
    
            
    
    alpha_Pi_KIPZ = 0.5*simple_integrate(KIPZ_v_ki[:,0], KIPZ_v_ki[:,1]*KIPZ_density[:,1])
    
    print("HOMO energy DFT with KIPZ-Density = {:10.8f} Hartree".format(HOMO_energy_KIPZ_DFT-bottom_of_potential))
    print("HOMO energy KIPZ                  = {:10.8f} Hartree".format(HOMO_energy_KIPZ-bottom_of_potential))
    print("Difference between the two        = {:10.7f} Hartree".format((HOMO_energy_KIPZ-bottom_of_potential)-(HOMO_energy_KIPZ_DFT-bottom_of_potential)))
    print("alpha*Pi_KIPZ                     = {:10.7f} Hartree".format(alpha_Pi_KIPZ))
    print()

if(compute_difference_to_KIPZ == True):
    
    rs = KI_xc_potential[1:,0] 
    exact_xc_potential            = np.zeros_like(rs)
    for i, r in enumerate(rs):
        exact_xc_potential[i]     = v_xc(rs[i])
  


if(results_exact_empty_states==True):
    print("ref. HOMO         = {:1.4f} Hartree".format(reference_HOMO))
    print("ref. LUMO         = {:1.4f} Hartree".format(reference_LUMO))
    print("ref. LUMO+1       = {:1.4f} Hartree".format(reference_LUMO_plus_one))
    print("ref. Total energy = {:1.4f} Hartree".format(reference_total_energy))
    print()

if(results_LDA_empty_states==True):
    eigenvalues_LDA  = np.loadtxt("../Analysis_LDA/postprocessing/results/eigenvalues.out")-bottom_of_potential
    total_energy_LDA = np.loadtxt("../Analysis_LDA/postprocessing/results/total_energy.out")/2.0-2*bottom_of_potential
    print("LDA HOMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_LDA[0], 100*(eigenvalues_LDA[0]-reference_HOMO)/reference_HOMO))
    print("LDA LUMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_LDA[1], 100*(eigenvalues_LDA[1]-reference_LUMO)/reference_LUMO))
    print("LDA LUMO+1        = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_LDA[4], 100*(eigenvalues_LDA[4]-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print("LDA Total energy  = {:1.4f} Hartree ({:+6.2f}%)".format(total_energy_LDA,   100*(total_energy_LDA-reference_total_energy)/reference_total_energy))
    print()

if(results_PBE_empty_states==True):
    eigenvalues_PBE  = np.loadtxt("../Analysis_PBE/postprocessing/results/eigenvalues.out")-bottom_of_potential
    total_energy_PBE = np.loadtxt("../Analysis_PBE/postprocessing/results/total_energy.out")/2-2*bottom_of_potential
    print("PBE HOMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PBE[0], 100*(eigenvalues_PBE[0]-reference_HOMO)/reference_HOMO))
    print("PBE LUMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PBE[1], 100*(eigenvalues_PBE[1]-reference_LUMO)/reference_LUMO))
    print("PBE LUMO+1        = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PBE[4], 100*(eigenvalues_PBE[4]-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print("PBE Total energy  = {:1.4f} Hartree ({:+6.2f}%)".format(total_energy_PBE,   100*(total_energy_PBE-reference_total_energy)/reference_total_energy))
    print()


if(results_DSCF_empty_states==True):
    total_energy_PBE   = np.loadtxt("../Analysis_PBE/postprocessing/results/total_energy.out")/2-2*bottom_of_potential
    DSCF_HOMO          = total_energy_PBE-(np.loadtxt("../Analysis_PBE_DSCF_HOMO/postprocessing/results/total_energy.out")/2-bottom_of_potential)
    DSCF_LUMO          = np.loadtxt("../Analysis_PBE_DSCF_LUMO/postprocessing/results/total_energy.out")/2-3*bottom_of_potential-total_energy_PBE
    DSCF_LUMO_plus_one = np.loadtxt("../Analysis_PBE_DSCF_LUMO_plus_1/postprocessing/results/total_energy.out")/2-3*bottom_of_potential-total_energy_PBE
    print("DSCF HOMO        = {:1.4f} Hartree ({:+6.2f}%)".format(DSCF_HOMO, 100*(DSCF_HOMO-reference_HOMO)/reference_HOMO))
    print("DSCF LUMO        = {:1.4f} Hartree ({:+6.2f}%)".format(DSCF_LUMO, 100*(DSCF_LUMO-reference_LUMO)/reference_LUMO))
    print("DSCF LUMO+1      = {:1.4f} Hartree ({:+6.2f}%)".format(DSCF_LUMO_plus_one, 100*(DSCF_LUMO_plus_one-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print()

if(results_PZ_empty_states==True):
    eigenvalues_PZ  = np.loadtxt("../Analysis_KIPZ_LUMO/postprocessing_pz/results/eigenvalues.out")-bottom_of_potential
    total_energy_PZ = np.loadtxt("../Analysis_KIPZ_LUMO/postprocessing_pz/results/total_energy.out")-2*bottom_of_potential
    print("PZ HOMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PZ[0], 100*(eigenvalues_PZ[0]-reference_HOMO)/reference_HOMO))
    print("PZ LUMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PZ[1], 100*(eigenvalues_PZ[1]-reference_LUMO)/reference_LUMO))
    print("PZ LUMO+1        = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_PZ[4], 100*(eigenvalues_PZ[4]-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print("PZ Total energy  = {:1.4f} Hartree ({:+6.2f}%)".format(total_energy_PZ,   100*(total_energy_PZ-reference_total_energy)/reference_total_energy))
    print()

if(results_KI_empty_states==True):
    eigenvalues_KI  = np.loadtxt("../Analysis_KI_LUMO/postprocessing/results/eigenvalues.out")-bottom_of_potential
    total_energy_KI = np.loadtxt("../Analysis_KI_LUMO/postprocessing/results/total_energy.out")-2*bottom_of_potential
    print("KI HOMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KI[0], 100*(eigenvalues_KI[0]-reference_HOMO)/reference_HOMO))
    print("KI LUMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KI[1], 100*(eigenvalues_KI[1]-reference_LUMO)/reference_LUMO))
    print("KI LUMO+1        = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KI[4], 100*(eigenvalues_KI[4]-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print("KI Total energy  = {:1.4f} Hartree ({:+6.2f}%)".format(total_energy_KI,   100*(total_energy_KI-reference_total_energy)/reference_total_energy))
    print()

if(results_KIPZ_empty_states==True):
    eigenvalues_KIPZ  = np.loadtxt("../Analysis_KIPZ_LUMO/postprocessing/results/eigenvalues.out")-bottom_of_potential
    total_energy_KIPZ = np.loadtxt("../Analysis_KIPZ_LUMO/postprocessing/results/total_energy.out")-2*bottom_of_potential
    print("KIPZ HOMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KIPZ[0], 100*(eigenvalues_KIPZ[0]-reference_HOMO)/reference_HOMO))
    print("KIPZ LUMO          = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KIPZ[1], 100*(eigenvalues_KIPZ[1]-reference_LUMO)/reference_LUMO))
    print("KIPZ LUMO+1        = {:1.4f} Hartree ({:+6.2f}%)".format(eigenvalues_KIPZ[4], 100*(eigenvalues_KIPZ[4]-reference_LUMO_plus_one)/reference_LUMO_plus_one))
    print("KIPZ Total energy  = {:1.4f} Hartree ({:+6.2f}%)".format(total_energy_KIPZ,   100*(total_energy_KIPZ-reference_total_energy)/reference_total_energy))
    print()
    
    
if(plot == True):

    fig, ax = plt.subplots(figsize=(fig_size, fig_size))
    plt.xlim((0, 12.0))
    plt.xlabel("Radial distance")
    plt.ylabel("Charge density")
    plt.grid(True)
    for i, result in enumerate(list_for_densities):
        np.savetxt("txt_files/" + str(i) + "_density_" + labels_for_densities[i] + "_" + linestyles_for_densities[i], result)
        plt.plot(result[:,0], result[:,1], label = labels_for_densities[i], linestyle = linestyles_for_densities[i])
    plt.legend()
    plt.savefig("plots/density.png", dpi = resolution)
    plt.show()



    fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(fig_size, fig_size))
    ax1.set_xlim((0, 12.0))
    # ax1.set_ylim((-0.8, 0.1))
    ax1.set_ylabel("$v_x(r)$")
    ax1.grid(True)
    for i, result in enumerate(list_for_x_pots):
        ax1.plot(result[:,0], result[:,1], label = labels_for_densities[i])
    # ax1.legend()
    # ax1.legend(loc='lower left', bbox_to_anchor=(0, 0.5))#, frameon=False, ncol=3)
    for ticklabel in ax1.xaxis.get_ticklabels():
        ticklabel.set_visible(False)
        
    ax2.set_xlim((0, 12.0))
    # ax2.set_ylim((-0.07, 0.02))
    ax2.set_xlabel("Radial distance")
    ax2.set_ylabel("$v_c(r)$")
    ax2.grid(True)
    for i, result in enumerate(list_for_c_pots):
        ax2.plot(result[:,0], result[:,1])#, label = labels_for_densities[i])
    # ax2.legend()
    fig.align_ylabels((ax1, ax2))
    plt.subplots_adjust(left_three_after_point, bottom, right, top)
    plt.figlegend(loc = 'lower left', bbox_to_anchor=(0.7, 0.46))
    plt.savefig("plots/exchange_and_correlation_potential.png", dpi=resolution)
    plt.show()
   

    # plt.figure(figsize = (fig_size, fig_size))
    # plt.xlim((0, 10.0))
    # plt.xlabel("Radial distance")
    # plt.ylabel("$v_{xc}(r)$")
    # plt.grid(True)
    # for i, result in enumerate(list_for_xc_pots):
    #     plt.plot(result[:,0], result[:,1], label = labels_for_densities[i], linestyle = linestyles_for_xc_potential[i])
    # plt.legend()
    # plt.subplots_adjust(left_two_after_point, bottom, right, top)
    # plt.savefig("plots/exchange-correlation_potential.png", dpi=resolution)
    # plt.show()
    

    fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(fig_size, fig_size))
    ax1.set_xlim((0, 12.15))
    ax1.set_ylabel("$v_{xc}(r)\phi_{HOMO}^{KS}(r)$")
    ax1.grid(True)
    for i, result in enumerate(list_for_xc_appl_pots):
        ax1.plot(result[:,0], result[:,1], label = labels_for_densities[i])
    # ax1.legend()
    for ticklabel in ax1.xaxis.get_ticklabels():
        ticklabel.set_visible(False)
        
    ax2.set_xlim((0, 12.15))
    ax2.set_xlabel("Radial distance")
    ax2.set_ylabel('$4\pi r^2v_{xc}(r)n_{HOMO}(r)$')
    ax2.grid(True)
    for i, result in enumerate(list_for_xc_appl_scaled_pots):
        ax2.plot(result[:,0], result[:,1])#, label = labels_for_densities[i])
    # ax2.legend()
    fig.align_ylabels((ax1, ax2))
    plt.subplots_adjust(left_three_after_point, bottom, right, top)
    plt.figlegend(loc = 'lower left', bbox_to_anchor=(0.7, 0.49))
    plt.savefig("plots/both_applied.png", dpi=resolution)
    plt.show()



    cycle = plt.rcParams['axes.prop_cycle'].by_key()['color']

    fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=(fig_size, fig_size))
    ax1.set_xlim((0, 24.3))
    ax1.set_ylabel("$v_{xc}(r)$")
    ax1.grid(True)
    for ticklabel in ax1.xaxis.get_ticklabels():
        ticklabel.set_visible(False)
    for i, result in enumerate(list_for_xc_pots):
        ax1.plot(result[:,0], result[:,1], label = labels_for_densities[i], linestyle = linestyles_for_xc_potential[i])

    ax2.set_xlim((0, 24.3))
    ax2.set_ylabel("$v_{xc}(r)-v_{xc}^{exact}(r)$")
    ax2.set_xlabel("Radial distance")
    ax2.grid(True)
    ax2.plot(rs, PZ_xc_potential[1:,1]  -exact_xc_potential, color=cycle[3])
    ax2.plot(rs, KI_xc_potential[1:,1]  -exact_xc_potential, color=cycle[4])
    ax2.plot(rs, KIPZ_xc_potential[1:,1]-exact_xc_potential, color=cycle[5], linestyle ='dashdot')
    # ax2.legend()#, frameon=False, ncol=3)

    fig.align_ylabels((ax1, ax2))
    plt.subplots_adjust(left_three_after_point, bottom, right, top)
    # ax1.legend(loc='lower left', bbox_to_anchor=(0.0, -0.5))
    plt.figlegend(loc = 'lower left', bbox_to_anchor=(0.7, 0.45))
    plt.savefig("plots/exchange-correlation_potential.png", dpi=resolution)
    plt.show()
