import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import FormatStrFormatter
import seaborn as sns
import os

# Specifications for nicer plots
resolution             = 1000
fig_size               = 246.0/72.27 
plt.rcParams['text.latex.preamble']=r"\usepackage{amsmath}"
sns.set(style = "whitegrid", context = "paper",  rc={'text.usetex' : True, "font.family": 'Times New Roman'}) #, "font.serif": "Computer Modern"})
sns.set_palette(sns.color_palette("bright"))


# omega = 0.1 -> 1
# omega = 0.5 -> 2
# omega =10.0 -> 3

path_1 = "../omega_one_tenth/Analysis/Analysis_8.5_30.0_20/postprocessing_all/txt_files/"
path_2 = "../omega_one_half/Analysis/Analysis_3.5_12.5_50/postprocessing_all/txt_files/"
path_3 = "../omega_ten/Analysis/Analysis_0.73_2.6_600/postprocessing_all/txt_files/"

paths = [path_1, path_2, path_3]
fig = plt.figure(figsize=(fig_size, fig_size))
ax_all = fig.add_subplot(1, 4, (2, 4))

vertical_between_figures = 0.08
vertical_offset = 0.02
height = (1-2*vertical_between_figures-2*vertical_offset)/3.0
starting_point = -0.44
width = 0.42

# position where the peak has dropped to 0.1 of its original height:
pos_1 = 8.078733724
pos_2 = 3.325494190
pos_3 = 0.695643500

width_2 = 0.55
maxs = np.zeros(3)

margin = 0.005
upper_mult = 1+margin
lower_mult = 1-margin

plt.grid(True)
for i, path in enumerate(paths):
    for j, file in enumerate(sorted(os.listdir(path))):
        result = np.loadtxt(path + file)
        label = file.split("_")[2]
        linestyle = file.split("_")[3]
        ax_all.semilogy(result[:,0], result[:,1], label = label, linestyle = linestyle)  
        if j==3: #PBE
            maxs[i] = result[0,1]
    if i == 0:
        ax_all.legend(loc='upper right')
    plt.gca().set_prop_cycle(None)
ax_all.set_ylim((5e-4, 15.0))
ax_all.set_xlim((0.0, 6.5))
ax_all.set_ylabel("Charge density")
ax_all.set_xlabel("Radial distance")
ax_all.tick_params(left=False, right=False, labelleft=False, labelright=True)
ax_all.yaxis.set_label_position("right")
plt.text(0.46, 4.6, r'$\boldsymbol{\omega=10}$', size=13)
plt.text(2.2, 0.019, r'$\boldsymbol{\omega=\frac{1}{2}}$', size=13)
plt.text(4.2, 0.0025, r'$\boldsymbol{\omega=\frac{1}{10}}$', size=13)

ax1 = ax_all.inset_axes([starting_point, vertical_offset+2*height+2*vertical_between_figures, width, height])
cut = 400
path = paths[2]
uppers = []
lowers = []
for file in sorted(os.listdir(path)):
    result = np.loadtxt(path + file)
    uppers.append(result[0,1])
    lowers.append(result[np.argmin(np.abs(pos_3/pos_2*width_2-result[:,0])),1])
    linestyle = file.split("_")[3]
    ax1.plot(result[:,0], result[:,1], linestyle = linestyle) 
ax1.set_xlim((0.0, pos_3/pos_2*width_2))
ax1.set_ylim((lower_mult*min(lowers), upper_mult*max(uppers)))
ax_all.indicate_inset_zoom(ax1, edgecolor="grey")
ax1.tick_params(axis='both', which='major', pad=0, labelcolor='grey')

ax2 = ax_all.inset_axes([starting_point, vertical_offset+height+vertical_between_figures, width, height])
path = paths[1]
uppers = []
lowers = []
for file in sorted(os.listdir(path)):
    result = np.loadtxt(path + file)
    uppers.append(result[0,1])
    lowers.append(result[np.argmin(np.abs(pos_2/pos_2*width_2-result[:,0])),1])
    linestyle = file.split("_")[3]
    ax2.plot(result[:,0], result[:,1], linestyle = linestyle) 
ax2.set_xlim((0.0, pos_2/pos_2*width_2))
ax2.set_ylim((lower_mult*min(lowers), upper_mult*max(uppers)))
ax_all.indicate_inset_zoom(ax2, edgecolor="grey")
ax2.tick_params(axis='both', which='major', pad=0, labelcolor='grey')

ax3 = ax_all.inset_axes([starting_point, vertical_offset, width, height])
path = paths[0]
uppers = []
lowers = []
for file in sorted(os.listdir(path)):
    result = np.loadtxt(path + file)
    uppers.append(result[0,1])
    lowers.append(result[np.argmin(np.abs(pos_1/pos_2*width_2-result[:,0])),1])
    linestyle = file.split("_")[3]
    ax3.plot(result[:,0], result[:,1], linestyle = linestyle)  
ax3.set_xlim((0.0, pos_1/pos_2*width_2))
# ax3.set_ylim((lower_mult*maxs[0], upper_mult*maxs[0]))
ax3.set_ylim((lower_mult*min(lowers), upper_mult*max(uppers)))
ax3.yaxis.set_major_locator(plt.MultipleLocator(0.001))
ax3.xaxis.set_major_formatter(FormatStrFormatter('%.1f'))
ax3.margins(x=0, y=0)
ax_all.indicate_inset_zoom(ax3, edgecolor="grey")
ax3.tick_params(axis='both', which='major', pad=0, labelcolor='grey')

plt.subplots_adjust(0.16, 0.13, 0.83, 0.98)
plt.grid(True)
plt.savefig("plots/density.png", dpi = resolution)
plt.show()
